@@ -9,6 +9,7 @@ use DBI qw(:utils);
 use JSON::PP;
 use Scalar::Util qw( blessed );
 use File::Slurp;
+use Locale::TextDomain qw[Zonemaster-Backend];
 
 use Zonemaster::LDNS;
 
@@ -16,6 +17,7 @@ use Zonemaster::Engine;
 use Zonemaster::Engine::Translator;
 use Zonemaster::Backend::Config;
 use Zonemaster::Engine::Profile;
+use Zonemaster::Engine::Util;
 
 sub new {
     my ( $class, $params ) = @_;
@@ -58,76 +60,19 @@ sub new {
 }
 
 sub run {
-    my ( $self, $test_id ) = @_;
+    my ( $self, $test_id, $show_progress ) = @_;
     my @accumulator;
-    my %counter;
-    my %counter_for_progress_indicator;
 
     my $params;
 
-    my $progress = $self->{_db}->test_progress( $test_id, 1 );
-
     $params = $self->{_db}->get_test_params( $test_id );
 
-    my %methods = Zonemaster::Engine->all_methods;
-
-    foreach my $module ( keys %methods ) {
-        foreach my $method ( @{ $methods{$module} } ) {
-            $counter_for_progress_indicator{planned}{ $module . '::' . $method } = $module . '::';
-        }
-    }
-
     my ( $domain ) = $params->{domain};
     if ( !$domain ) {
         die "Must give the name of a domain to test.\n";
     }
     $domain = $self->to_idn( $domain );
 
-    # used for progress indicator
-    my ( $previous_module, $previous_method ) = ( '', '' );
-
-    # Callback defined here so it closes over the setup above.
-    Zonemaster::Engine->logger->callback(
-        sub {
-            my ( $entry ) = @_;
-
-            foreach my $trace ( reverse @{ $entry->trace } ) {
-                foreach my $module_method ( keys %{ $counter_for_progress_indicator{planned} } ) {
-                    if ( index( $trace->[1], $module_method ) > -1 ) {
-                        my $percent_progress = 0;
-                        my ( $module ) = ( $module_method =~ /(.+::)[^:]+/ );
-                        if ( $previous_module eq $module ) {
-                            $counter_for_progress_indicator{executed}{$module_method}++;
-                        }
-                        elsif ( $previous_module ) {
-                            foreach my $planned_module_method ( keys %{ $counter_for_progress_indicator{planned} } ) {
-                                $counter_for_progress_indicator{executed}{$planned_module_method}++
-                                  if ( $counter_for_progress_indicator{planned}{$planned_module_method} eq
-                                    $previous_module );
-                            }
-                        }
-                        $previous_module = $module;
-
-                        if ( $previous_method ne $module_method ) {
-                            $percent_progress = sprintf(
-                                "%.0f",
-                                99 * (
-                                    scalar( keys %{ $counter_for_progress_indicator{executed} } ) /
-                                      scalar( keys %{ $counter_for_progress_indicator{planned} } )
-                                )
-                            );
-                            $self->{_db}->test_progress( $test_id, $percent_progress );
-
-                            $previous_method = $module_method;
-                        }
-                    }
-                }
-            }
-
-            $counter{ uc $entry->level } += 1;
-        }
-    );
-
     if ( $params->{nameservers} && @{ $params->{nameservers} } > 0 ) {
         $self->add_fake_delegation( $domain, $params->{nameservers} );
     }
@@ -136,7 +81,6 @@ sub run {
         $self->add_fake_ds( $domain, $params->{ds_info} );
     }
 
-
     # If the profile parameter has been set in the API, then load a profile
     if ( $params->{profile} ) {
         $params->{profile} = lc($params->{profile});
@@ -158,6 +102,34 @@ sub run {
         Zonemaster::Engine::Profile->effective->set( q{net.ipv6}, ( $params->{ipv6} ) ? ( 1 ) : ( 0 ) );
     }
 
+    if ( $show_progress ) {
+        my %methods = Zonemaster::Engine->all_methods;
+
+        # BASIC methods are always run: Basic0{0..4}
+        my $nbr_testcases_planned = 5;
+        my $nbr_testcases_finished = 0;
+
+        foreach my $module ( keys %methods ) {
+            foreach my $method ( @{ $methods{$module} } ) {
+                if ( Zonemaster::Engine::Util::should_run_test( $method ) ) {
+                    $nbr_testcases_planned++;
+                }
+            }
+        }
+
+        Zonemaster::Engine->logger->callback(
+            sub {
+                my ( $entry ) = @_;
+                if ( $entry->{tag} and $entry->{tag} eq 'TEST_CASE_END' ) {
+                    $nbr_testcases_finished++;
+                    # limit to max 99%, 100% is reached when data is stored in database
+                    my $progress_percent = int( 99 * $nbr_testcases_finished /  $nbr_testcases_planned );
+                    $self->{_db}->test_progress( $test_id, $progress_percent );
+                }
+            }
+        );
+    }
+
     # Actually run tests!
     eval { Zonemaster::Engine->test_zone( $domain ); };
     if ( $@ ) {
@@ -170,9 +142,7 @@ sub run {
         }
     }
 
-    $self->{_db}->test_results( $test_id, Zonemaster::Engine->logger->json( 'INFO' ) );
-
-    $progress = $self->{_db}->test_progress( $test_id );
+    $self->{_db}->store_results( $test_id, Zonemaster::Engine->logger->json( 'INFO' ) );
 
     return;
 } ## end sub run
@@ -234,7 +204,7 @@ sub to_idn {
         return Zonemaster::LDNS::to_idn( $str );
     }
     else {
-        warn __( "Warning: Zonemaster::LDNS not compiled with libidn, cannot handle non-ASCII names correctly." );
+        warn __( "Warning: Zonemaster::LDNS not compiled with IDN support, cannot handle non-ASCII names correctly." );
         return $str;
     }
 }
